@@ -209,10 +209,12 @@ gail_expander_new (GtkWidget *widget)
 static G_CONST_RETURN gchar*
 gail_expander_get_name (AtkObject *obj)
 {
+  G_CONST_RETURN gchar *name;
   g_return_val_if_fail (GAIL_IS_EXPANDER (obj), NULL);
 
-  if (obj->name != NULL)
-    return obj->name;
+  name = ATK_OBJECT_CLASS (parent_class)->get_name (obj);
+  if (name != NULL)
+    return name;
   else
     {
       /*
@@ -366,7 +368,7 @@ gail_expander_do_action (AtkAction *action,
   expander = GAIL_EXPANDER (action);
   switch (i)
     {
-    case 1:
+    case 0:
       if (expander->action_idle_handler)
         return_value = FALSE;
       else
