@@ -48,6 +48,7 @@ static gboolean              gail_window_real_focus_gtk (GtkWidget     *widget,
                                                          GdkEventFocus *event);
 
 static AtkStateSet*          gail_window_ref_state_set  (AtkObject     *accessible);
+static AtkRelationSet*       gail_window_ref_relation_set  (AtkObject     *accessible);
 static void                  gail_window_real_notify_gtk (GObject      *obj,
                                                           GParamSpec   *pspec);
 static gint                  gail_window_get_mdi_zorder (AtkComponent  *component);
@@ -127,6 +128,7 @@ gail_window_class_init (GailWindowClass *klass)
   class->get_name = gail_window_get_name;
   class->get_parent = gail_window_get_parent;
   class->get_index_in_parent = gail_window_get_index_in_parent;
+  class->ref_relation_set = gail_window_ref_relation_set;
   class->ref_state_set = gail_window_ref_state_set;
   class->initialize = gail_window_real_initialize;
 
@@ -296,10 +298,10 @@ gail_window_finalize (GObject *object)
 static G_CONST_RETURN gchar*
 gail_window_get_name (AtkObject *accessible)
 {
-  if (accessible->name)
-    return accessible->name;
+  G_CONST_RETURN gchar* name;
 
-  else
+  name = ATK_OBJECT_CLASS (parent_class)->get_name (accessible);
+  if (name == NULL)
     {
       /*
        * Get the window title if it exists
@@ -317,25 +319,20 @@ gail_window_get_name (AtkObject *accessible)
       if (GTK_IS_WINDOW (widget))
         {
           GtkWindow *window = GTK_WINDOW (widget);
-          G_CONST_RETURN gchar* name;
  
           name = gtk_window_get_title (window);
-          if (name)
-            return name;
-
-          if (accessible->role == ATK_ROLE_TOOL_TIP)
+          if (name == NULL &&
+              accessible->role == ATK_ROLE_TOOL_TIP)
             {
               GtkWidget *child;
 
               child = gtk_bin_get_child (GTK_BIN (window));
               g_return_val_if_fail (GTK_IS_LABEL (child), NULL);
               name = gtk_label_get_text (GTK_LABEL (child));
-              return name;
             }
         }
-                    
-      return ATK_OBJECT_CLASS (parent_class)->get_name (accessible);
     }
+  return name;
 }
 
 static AtkObject*
@@ -389,6 +386,49 @@ gail_window_real_focus_gtk (GtkWidget     *widget,
   return FALSE;
 }
 
+static AtkRelationSet*
+gail_window_ref_relation_set (AtkObject *obj)
+{
+  GtkWidget *widget;
+  AtkRelationSet *relation_set;
+  GtkWidget *label;
+  AtkObject *array[1];
+  AtkRelation* relation;
+  GtkWidget *current_widget;
+  gboolean ret;
+
+  g_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
+
+  widget = GTK_ACCESSIBLE (obj)->widget;
+  if (widget == NULL)
+    /*
+     * State is defunct
+     */
+    return NULL;
+
+  relation_set = ATK_OBJECT_CLASS (parent_class)->ref_relation_set (obj);
+
+  if (atk_object_get_role (obj) == ATK_ROLE_TOOL_TIP)
+    {
+      relation = atk_relation_set_get_relation_by_type (relation_set, ATK_RELATION_POPUP_FOR);
+
+      if (relation)
+        {
+          atk_relation_set_remove (relation_set, relation);
+        }
+      ret = gtk_tooltips_get_info_from_tip_window (GTK_WINDOW (widget), NULL, &current_widget);
+      if (ret)
+        {
+          array [0] = gtk_widget_get_accessible (current_widget);
+
+          relation = atk_relation_new (array, 1, ATK_RELATION_POPUP_FOR);
+          atk_relation_set_add (relation_set, relation);
+          g_object_unref (relation);
+        }
+    }
+  return relation_set;
+}
+
 static AtkStateSet*
 gail_window_ref_state_set (AtkObject *accessible)
 {
